@@ -1,10 +1,10 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1998-2016 Stuart Parmenter and others, see AUTHORS file.
+ * Copyright (C) 2013-2020 Peter Bloomfield
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,7 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 /*
@@ -34,7 +34,7 @@
 GtkWidget *libbalsa_window_get_menu_bar(GtkApplicationWindow * window,
                                         const GActionEntry   * entries,
                                         gint                   n_entries,
-                                        const gchar          * ui_file,
+                                        const gchar          * resource_path,
                                         GError              ** error,
                                         gpointer               cb_data);
 
