@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *  
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 /*
@@ -429,8 +429,7 @@ lbh_info_bar(LibBalsaWebKitInfo * info)
                                    (gtk_info_bar_get_action_area
                                     (info_bar)), GTK_ORIENTATION_VERTICAL);
 
-    label = gtk_label_new(_(text));
-    gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
+    label = libbalsa_create_wrap_label(text, FALSE);
 
     content_area = gtk_info_bar_get_content_area(info_bar);
     gtk_container_add(GTK_CONTAINER(content_area), label);
@@ -570,8 +569,8 @@ lbh_cid_cb(WebKitURISchemeRequest * request,
             mime_type = libbalsa_message_body_get_mime_type(body);
             webkit_uri_scheme_request_finish(request, stream, len,
                                              mime_type);
-            g_object_unref(stream);
             g_free(mime_type);
+            g_object_unref(stream);
         }
     }
 }
@@ -703,6 +702,7 @@ libbalsa_html_print_bitmap(LibBalsaMessageBody *body,
     have_src_oth = g_regex_match_simple(SRC_REGEX, text, G_REGEX_CASELESS, 0);
 
     info = g_new0(LibBalsaWebKitInfo, 1);
+    info->body = body;
 	offline_window = gtk_offscreen_window_new();
 	render_width = (gint) (width * HTML_PRINT_DPI / 72.0);
 	g_debug("%s: request Cairo width %g, render width %d", __func__, width, render_width);
@@ -1180,11 +1180,11 @@ libbalsa_html_filter(LibBalsaHTMLType html_type, gchar ** text, guint len)
 LibBalsaHTMLType
 libbalsa_html_type(const gchar * mime_type)
 {
-    if (!strcmp(mime_type, "text/html"))
+    if (strcmp(mime_type, "text/html") == 0)
 	return LIBBALSA_HTML_TYPE_HTML;
-    if (!strcmp(mime_type, "text/enriched"))
+    if (strcmp(mime_type, "text/enriched") == 0)
 	return LIBBALSA_HTML_TYPE_ENRICHED;
-    if (!strcmp(mime_type, "text/richtext"))
+    if (strcmp(mime_type, "text/richtext") == 0)
 	return LIBBALSA_HTML_TYPE_RICHTEXT;
     return LIBBALSA_HTML_TYPE_NONE;
 }
@@ -1194,7 +1194,7 @@ libbalsa_html_type(const gchar * mime_type)
 LibBalsaHTMLType
 libbalsa_html_type(const gchar * mime_type)
 {
-    if (!strcmp(mime_type, "text/html"))
+    if (strcmp(mime_type, "text/html") == 0)
 	return LIBBALSA_HTML_TYPE_HTML;
     return LIBBALSA_HTML_TYPE_NONE;
 }
