@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *  
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __LIBBALSA_MAILBOX_LOCAL_H__
@@ -67,17 +67,10 @@ LibBalsaMailbox *libbalsa_mailbox_local_new(const gchar * path,
                                             gboolean      create);
 gint libbalsa_mailbox_local_set_path(LibBalsaMailboxLocal * mailbox,
 				     const gchar * path, gboolean create);
-void libbalsa_mailbox_local_set_threading_info(LibBalsaMailboxLocal *
-                                               local);
 
 #define libbalsa_mailbox_local_get_path(local) \
 	(libbalsa_mailbox_get_url((LibBalsaMailbox*)local)+7)
 
-void libbalsa_mailbox_local_load_messages(LibBalsaMailbox * mailbox,
-					  guint last_msgno);
-void libbalsa_mailbox_local_cache_message(LibBalsaMailboxLocal * local,
-                                          guint msgno,
-                                          LibBalsaMessage * message);
 void libbalsa_mailbox_local_msgno_removed(LibBalsaMailbox * mailbox,
 					  guint msgno);
 void libbalsa_mailbox_local_remove_files(LibBalsaMailboxLocal *mailbox);
