@@ -894,9 +894,7 @@ lbm_maildir_add_message(LibBalsaMailboxLocal * local,
     out_stream = g_mime_stream_fs_new(fd);
 
     in_stream = g_mime_stream_filter_new(stream);
-    crlffilter =
-        g_mime_filter_crlf_new(FALSE,
-                               FALSE);
+    crlffilter = g_mime_filter_dos2unix_new(FALSE);
     g_mime_stream_filter_add(GMIME_STREAM_FILTER(in_stream), crlffilter);
     g_object_unref(crlffilter);
  
