@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *  
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __MAIN_WINDOW_H__
@@ -102,15 +102,14 @@ gboolean balsa_window_next_unread(BalsaWindow * window);
 gboolean balsa_window_setup_progress(BalsaWindow * window,
                                      const gchar * text);
 void balsa_window_clear_progress(BalsaWindow* window);
-void balsa_window_increment_progress(BalsaWindow * window,
-                                     gdouble fraction, gboolean flush);
+void balsa_window_progress_bar_set_fraction(BalsaWindow * window,
+                                            gdouble       fraction);
 void balsa_window_increase_activity(BalsaWindow * window,
                                     const gchar * message);
 void balsa_window_decrease_activity(BalsaWindow * window,
                                     const gchar * message);
 void balsa_window_set_statusbar(BalsaWindow     * window,
                                 LibBalsaMailbox * mailbox);
-void balsa_window_set_thread_messages(BalsaWindow * window, gboolean thread_messages);
 
 /*
  * Getter
