@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
@@ -66,7 +66,7 @@ static gboolean cmd_check_mail_on_startup,
 /* opt_attach_list: list of attachments */
 static gchar **opt_attach_list = NULL;
 /* opt_compose_email: To: field for the compose window */
-static gchar *opt_compose_email = NULL;
+static const gchar *opt_compose_email = NULL;
 
 static void
 accel_map_load(void)
@@ -134,9 +134,9 @@ check_special_mailboxes(void)
 }
 
 static void
-config_init(gboolean check_only)
+config_init(void)
 {
-    while(!config_load() && !check_only) {
+    while (!config_load()) {
 	balsa_init_begin();
         config_defclient_save();
     }
@@ -193,17 +193,21 @@ initial_open_inbox()
 static void
 balsa_get_stats(long *unread, long *unsent)
 {
-
     if (balsa_app.inbox && libbalsa_mailbox_open(balsa_app.inbox, NULL)) {
         /* set threading type to load messages */
         libbalsa_mailbox_set_threading(balsa_app.inbox);
         *unread = libbalsa_mailbox_get_unread_messages(balsa_app.inbox);
         libbalsa_mailbox_close(balsa_app.inbox, FALSE);
-    } else *unread = -1;
-    if(balsa_app.draftbox && libbalsa_mailbox_open(balsa_app.outbox, NULL)){
+    } else {
+        *unread = -1;
+    }
+
+    if (balsa_app.outbox && libbalsa_mailbox_open(balsa_app.outbox, NULL)){
         *unsent = libbalsa_mailbox_total_messages(balsa_app.outbox);
         libbalsa_mailbox_close(balsa_app.outbox, FALSE);
-    } else *unsent = -1;
+    } else {
+        *unsent = -1;
+    }
 }
 
 static gboolean
@@ -220,7 +224,7 @@ balsa_check_open_mailboxes(void)
     gchar **urls;
 
     join = g_strjoinv(";", cmd_line_open_mailboxes);
-    g_strfreev(cmd_line_open_mailboxes);
+    g_free(cmd_line_open_mailboxes); /* It was a shallow copy. */
     cmd_line_open_mailboxes = NULL;
 
     urls = g_strsplit(join, ";", 20);
@@ -265,13 +269,13 @@ scan_mailboxes_idle_cb()
         g_list_free(gl);
     }
 
-    if (cmd_line_open_mailboxes) {
+    if (cmd_line_open_mailboxes != NULL) {
         gchar *join;
         gchar **urls;
         gchar **p;
 
         join = g_strjoinv(";", cmd_line_open_mailboxes);
-        g_strfreev(cmd_line_open_mailboxes);
+        g_free(cmd_line_open_mailboxes); /* It was a shallow copy. */
         cmd_line_open_mailboxes = NULL;
 
         urls = g_strsplit(join, ";", 20);
@@ -299,7 +303,7 @@ scan_mailboxes_idle_cb()
                                                             FALSE));
     }
 
-    if(cmd_get_stats) {
+    if (cmd_get_stats) {
         long unread, unsent;
         balsa_get_stats(&unread, &unsent);
         printf("Unread: %ld Unsent: %ld\n", unread, unsent);
@@ -416,8 +420,7 @@ balsa_progress_set_fraction(LibBalsaProgress * progress, gdouble fraction)
     min_fraction  += LIBBALSA_PROGRESS_MIN_UPDATE_STEP;
 
     if (balsa_app.main_window)
-        balsa_window_increment_progress(balsa_app.main_window, fraction,
-                                        !libbalsa_am_i_subthread());
+        balsa_window_progress_bar_set_fraction(balsa_app.main_window, fraction);
 }
 
 static void
@@ -434,26 +437,25 @@ balsa_progress_set_activity(gboolean set, const gchar * text)
 static gboolean
 balsa_check_open_compose_window(void)
 {
-    if (opt_compose_email || opt_attach_list) {
+    if (opt_compose_email != NULL || opt_attach_list != NULL) {
         BalsaSendmsg *snd;
         gchar **attach;
 
         snd = sendmsg_window_compose();
         snd->quit_on_close = FALSE;
 
-        if (opt_compose_email) {
+        if (opt_compose_email != NULL) {
             if (g_ascii_strncasecmp(opt_compose_email, "mailto:", 7) == 0)
                 sendmsg_window_process_url(snd, opt_compose_email + 7, TRUE);
             else
                 sendmsg_window_set_field(snd, "to", opt_compose_email, TRUE);
-            g_free(opt_compose_email);
             opt_compose_email = NULL;
         }
 
-        if (opt_attach_list) {
+        if (opt_attach_list != NULL) {
             for (attach = opt_attach_list; *attach; ++attach)
                 add_attachment(snd, *attach, FALSE, NULL);
-            g_strfreev(opt_attach_list);
+            g_free(opt_attach_list); /* It was a shallow copy. */
             opt_attach_list = NULL;
         }
 
@@ -545,23 +547,12 @@ balsa_startup_cb(GApplication *application,
     }
 #endif
 
-    /* checking for valid config files */
-    config_init(cmd_get_stats);
-
     default_icon = balsa_pixmap_finder("balsa_icon.png");
-    if(default_icon) { /* may be NULL for developer installations */
+    if (default_icon) { /* may be NULL for developer installations */
         gtk_window_set_default_icon_from_file(default_icon, NULL);
         g_free(default_icon);
     }
 
-    if (cmd_get_stats) {
-        long unread, unsent;
-        balsa_get_stats(&unread, &unsent);
-        printf("Unread: %ld Unsent: %ld\n", unread, unsent);
-        g_application_quit(G_APPLICATION(balsa_app.application));
-        return;
-    }
-
     balsa_app.has_openpgp =
         libbalsa_gpgme_check_crypto_engine(GPGME_PROTOCOL_OpenPGP);
     balsa_app.has_smime =
@@ -573,6 +564,9 @@ balsa_startup_cb(GApplication *application,
 static void
 balsa_shutdown_cb(void)
 {
+    if (cmd_get_stats)
+        return;
+
     balsa_app_destroy();
 
     libbalsa_conf_drop_all();
@@ -629,93 +623,51 @@ balsa_activate_cb(GApplication *application,
 /*
  * Parse command line options
  */
-static gint
-parse_options(int                       argc,
-              char                   ** argv,
-              GApplicationCommandLine * command_line)
+static GOptionEntry option_entries[] = {
+    {"check-mail", 'c', 0, G_OPTION_ARG_NONE, NULL,
+        N_("Get new mail on start-up"), NULL},
+    {"compose", 'm', 0, G_OPTION_ARG_STRING, NULL,
+        N_("Compose a new email to EMAIL@ADDRESS"), "EMAIL@ADDRESS"},
+    {"attach", 'a', 0, G_OPTION_ARG_FILENAME_ARRAY, NULL,
+        N_("Attach file at URI"), "URI"},
+    {"open-mailbox", 'o', 0, G_OPTION_ARG_STRING_ARRAY, NULL,
+        N_("Opens MAILBOXNAME"), N_("MAILBOXNAME")},
+    {"open-unread-mailbox", 'u', 0, G_OPTION_ARG_NONE, NULL,
+        N_("Opens first unread mailbox"), NULL},
+    {"open-inbox", 'i', 0, G_OPTION_ARG_NONE, NULL,
+        N_("Opens default Inbox on start-up"), NULL},
+    {"get-stats", 's', 0, G_OPTION_ARG_NONE, NULL,
+        N_("Prints number unread and unsent messages"), NULL},
+    {"version", 'v', 0, G_OPTION_ARG_NONE, NULL,
+        N_("Show version"), NULL},
+    /* last but not least a special option that collects filenames */
+    {G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY, NULL,
+        "Special option that collects any remaining arguments for us"},
+    {NULL}
+};
+
+static void
+parse_options(GApplicationCommandLine * cmdline)
 {
-    static gboolean help;
-    static gboolean version;
-    static gchar **remaining_args;
-    static GOptionEntry option_entries[] = {
-        {"checkmail", 'c', 0, G_OPTION_ARG_NONE,
-         &(cmd_check_mail_on_startup),
-         N_("Get new mail on start-up"), NULL},
-        {"compose", 'm', 0, G_OPTION_ARG_STRING, &(opt_compose_email),
-         N_("Compose a new email to EMAIL@ADDRESS"), "EMAIL@ADDRESS"},
-        {"attach", 'a', 0, G_OPTION_ARG_FILENAME_ARRAY, &(opt_attach_list),
-         N_("Attach file at URI"), "URI"},
-        {"open-mailbox", 'o', 0, G_OPTION_ARG_STRING_ARRAY,
-         &(cmd_line_open_mailboxes),
-         N_("Opens MAILBOXNAME"), N_("MAILBOXNAME")},
-        {"open-unread-mailbox", 'u', 0, G_OPTION_ARG_NONE,
-         &(cmd_open_unread_mailbox),
-         N_("Opens first unread mailbox"), NULL},
-        {"open-inbox", 'i', 0, G_OPTION_ARG_NONE,
-         &(cmd_open_inbox),
-         N_("Opens default Inbox on start-up"), NULL},
-        {"get-stats", 's', 0, G_OPTION_ARG_NONE,
-         &(cmd_get_stats),
-         N_("Prints number unread and unsent messages"), NULL},
-        {"help", 'h', 0, G_OPTION_ARG_NONE, &help, N_("Show help options"), NULL},
-        {"version", 'v', 0, G_OPTION_ARG_NONE, &version, N_("Show version"), NULL},
-        /* last but not least a special option that collects filenames */
-        {G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY,
-         &remaining_args,
-         "Special option that collects any remaining arguments for us"},
-        {NULL}
-    };
-    GOptionContext *context;
-    gboolean rc;
-    GError *error;
-    gint status = 0;
+    GVariantDict *dict = g_application_command_line_get_options_dict(cmdline);
+    gchar **remaining_args;
 
-    context = g_option_context_new(NULL);
-    g_option_context_add_main_entries(context, option_entries, NULL);
+    cmd_check_mail_on_startup = g_variant_dict_contains(dict, "check-mail");
+    cmd_open_unread_mailbox = g_variant_dict_contains(dict, "open-unread-mailbox");
+    cmd_open_inbox = g_variant_dict_contains(dict, "open-inbox");
+    cmd_get_stats = g_variant_dict_contains(dict, "get-stats");
 
-    /* Disable the built-in help-handling of GOptionContext, since it
-     * calls exit() after printing help, which is not what we want to
-     * happen in the primary instance. */
-    g_option_context_set_help_enabled(context, FALSE);
+    if (!g_variant_dict_lookup(dict, "compose", "&s", &opt_compose_email))
+        opt_compose_email = NULL;
 
-    cmd_check_mail_on_startup = FALSE;
-    cmd_get_stats = FALSE;
-    cmd_open_unread_mailbox = FALSE;
-    cmd_open_inbox = FALSE;
-    help = FALSE;
-    version = FALSE;
-    remaining_args = NULL;
-    error = NULL;
+    if (!g_variant_dict_lookup(dict, "attach", "^a&ay", &opt_attach_list))
+        opt_attach_list = NULL;
 
-    rc = g_option_context_parse(context, &argc, &argv, &error);
-    /* We offer the usual --help and -h as help options, but we should
-     * also support the legacy "-?".
-     * If we got an error, we check to see if it was caused by -?,
-     * and if so, honor it: */
-    if (!rc && strcmp(*++argv, "-?") == 0) {
-        rc = help = TRUE;
-        g_error_free(error);
-    }
+    if (!g_variant_dict_lookup(dict, "open-mailbox", "^a&s", &cmd_line_open_mailboxes))
+        cmd_line_open_mailboxes = NULL;
 
-    if (!rc) {
-        /* Some other bad option */
-        g_application_command_line_printerr(command_line, "%s\n",
-                                            error->message);
-        g_error_free(error);
-        status = 1;
-    } else if (help) {
-        gchar *text;
-
-        text = g_option_context_get_help(context, FALSE, NULL);
-        g_application_command_line_print(command_line, "%s", text);
-        g_free(text);
-        status = 2;
-    } else if (version) {
-        g_application_command_line_print(command_line,
-                                         "Balsa email client %s\n",
-                                         BALSA_VERSION);
-        status = 2;
-    }
+    if (!g_variant_dict_lookup(dict, G_OPTION_REMAINING, "^a&ay", &remaining_args))
+        remaining_args = NULL;
 
     if (remaining_args != NULL) {
         gint i, num_args;
@@ -725,18 +677,13 @@ parse_options(int                       argc,
             /* process remaining_args[i] here */
             /* we do nothing for now */
         }
-        g_strfreev(remaining_args);
+        g_free(remaining_args); /* It was a shallow copy. */
         remaining_args = NULL;
     }
-
-    g_option_context_free(context);
-
-    return status;
 }
 
 static void
-handle_remote(int argc, char **argv,
-              GApplicationCommandLine * command_line)
+handle_remote(GApplicationCommandLine * command_line)
 {
     if (cmd_get_stats) {
         glong unread, unsent;
@@ -755,7 +702,7 @@ handle_remote(int argc, char **argv,
         if (cmd_open_inbox)
             initial_open_inbox();
 
-        if (cmd_line_open_mailboxes)
+        if (cmd_line_open_mailboxes != NULL)
             balsa_check_open_mailboxes();
 
         if (!balsa_check_open_compose_window()) {
@@ -766,39 +713,51 @@ handle_remote(int argc, char **argv,
     }
 }
 
+static gint
+balsa_handle_local_options_cb(GApplication *application,
+                              GVariantDict *options,
+                              gpointer      user_data)
+{
+    if (g_variant_dict_contains(options, "version")) {
+        g_print("Balsa email client %s\n", BALSA_VERSION);
+
+        return 0;
+    }
+
+    return -1;
+}
+
 static int
 balsa_command_line_cb(GApplication            * application,
-                GApplicationCommandLine * command_line,
-                gpointer                  user_data)
+                      GApplicationCommandLine * command_line,
+                      gpointer                  user_data)
 {
-    gchar **args, **argv;
-    gint argc;
-    int status;
-
-    args = g_application_command_line_get_arguments(command_line, &argc);
-    /* We have to make an extra copy of the array, since
-     * g_option_context_parse() assumes that it can remove strings from
-     * the array without freeing them. */
-    argv = g_memdup(args, (argc + 1) * sizeof(gchar *));
-
     /* The signal is emitted when the GApplication is run, but is always
      * handled by the primary instance of Balsa. */
-    status = parse_options(argc, argv, command_line);
-    if (status == 0) {
-        if (g_application_command_line_get_is_remote(command_line)) {
-            /* A remote instance caused the emission; skip start-up, just
-             * handle the command line. */
-            handle_remote(argc, argv, command_line);
-        } else {
-            g_application_activate(application);
-        }
-    } else if (status == 2) /* handled a "help" or "version" request */
-        status = 0;
+    parse_options(command_line);
 
-    g_free(argv);
-    g_strfreev(args);
+    if (g_application_command_line_get_is_remote(command_line)) {
+        /* A remote instance caused the emission; skip start-up, just
+         * handle the command line. */
+        handle_remote(command_line);
+    } else if (cmd_get_stats) {
+        long unread, unsent;
 
-    return status;
+        balsa_app.inbox =
+            libbalsa_mailbox_new_from_config("mailbox-Inbox", FALSE);
+        balsa_app.outbox =
+            libbalsa_mailbox_new_from_config("mailbox-Outbox", FALSE);
+        balsa_get_stats(&unread, &unsent);
+        printf("Unread: %ld Unsent: %ld\n", unread, unsent);
+        g_object_unref(balsa_app.outbox);
+        g_object_unref(balsa_app.inbox);
+    } else {
+        /* checking for valid config files */
+        config_init();
+        g_application_activate(application);
+    }
+
+    return 0;
 }
 
 int
@@ -811,7 +770,10 @@ main(int argc, char **argv)
         gtk_application_new("org.desktop.Balsa",
                             G_APPLICATION_HANDLES_COMMAND_LINE);
     g_object_set(application, "register-session", TRUE, NULL);
+    g_application_add_main_option_entries(G_APPLICATION(application), option_entries);
 
+    g_signal_connect(application, "handle-local-options",
+                     G_CALLBACK(balsa_handle_local_options_cb), NULL);
     g_signal_connect(application, "command-line",
                      G_CALLBACK(balsa_command_line_cb), NULL);
     g_signal_connect(application, "startup",
